@@ -7,7 +7,8 @@
 ##
 
 import re
-from   capstone import *
+
+from capstone import *
 
 
 class Gadgets(object):
@@ -30,7 +31,7 @@ class Gadgets(object):
         self.__filterRE = re.compile("({})$".format(re_str)) if re_str else None
 
     def __passCleanX86(self, decodes):
-        br = ["ret", "retf", "int", "sysenter", "jmp", "call", "syscall"]
+        br = ["ret", "repz ret", "retf", "int", "sysenter", "jmp", "notrack jmp", "call", "notrack call", "syscall"]
 
         if decodes[-1][2] not in br:
             return True
@@ -43,7 +44,7 @@ class Gadgets(object):
 
     def __gadgetsFinding(self, section, gadgets, arch, mode):
 
-        PREV_BYTES = 9 # Number of bytes prior to the gadget to store.
+        PREV_BYTES = 9  # Number of bytes prior to the gadget to store.
 
         opcodes = section["opcodes"]
         sec_vaddr = section["vaddr"]
@@ -51,29 +52,31 @@ class Gadgets(object):
         ret = []
         md = Cs(arch, mode)
         for gad_op, gad_size, gad_align in gadgets:
+            if self.__options.align:
+                gad_align = self.__options.align
             allRefRet = [m.start() for m in re.finditer(gad_op, opcodes)]
             for ref in allRefRet:
                 end = ref + gad_size
                 for i in range(self.__options.depth):
                     start = ref - (i * gad_align)
-                    if (sec_vaddr+start) % gad_align == 0:
+                    if (sec_vaddr + start) % gad_align == 0:
                         code = opcodes[start:end]
-                        decodes = md.disasm_lite(code, sec_vaddr+ref)
+                        decodes = md.disasm_lite(code, sec_vaddr + start)
                         decodes = list(decodes)
-                        if sum(size for _, size, _, _ in decodes) != i*gad_align + gad_size:
+                        if sum(size for _, size, _, _ in decodes) != i * gad_align + gad_size:
                             # We've read less instructions than planned so something went wrong
                             continue
                         if self.passClean(decodes):
                             continue
                         off = self.__offset
-                        vaddr = off+sec_vaddr+start
-                        g = {"vaddr" :  vaddr}
+                        vaddr = off + sec_vaddr + start
+                        g = {"vaddr": vaddr}
                         if not self.__options.noinstr:
                             g["gadget"] = " ; ".join("{}{}{}".format(mnemonic, " " if op_str else "", op_str)
                                                      for _, _, mnemonic, op_str in decodes).replace("  ", " ")
                         if self.__options.callPreceded:
                             prevBytesAddr = max(sec_vaddr, vaddr - PREV_BYTES)
-                            g["prev"] = opcodes[prevBytesAddr-sec_vaddr:vaddr-sec_vaddr]
+                            g["prev"] = opcodes[prevBytesAddr - sec_vaddr:vaddr - sec_vaddr]
                         if self.__options.dump:
                             g["bytes"] = code
                         ret.append(g)
@@ -92,11 +95,12 @@ class Gadgets(object):
                             [b"\xcb", 1, 1],                # retf
                             [b"\xca[\x00-\xff]{2}", 3, 1],  # retf <imm>
                             # MPX
-                            [b"\xf2\xc3", 2, 1],               # ret
-                            [b"\xf2\xc2[\x00-\xff]{2}", 4, 1], # ret <imm>
+                            [b"\xf2\xc3", 2, 1],                # ret
+                            [b"\xf2\xc2[\x00-\xff]{2}", 4, 1],  # ret <imm>
                        ]
 
-        elif arch == CS_ARCH_MIPS:   gadgets = []            # MIPS doesn't contains RET instruction set. Only JOP gadgets
+        elif arch == CS_ARCH_MIPS:
+            gadgets = []  # MIPS doesn't have RET instructions. Only JOP gadgets
         elif arch == CS_ARCH_PPC:
             if arch_endian == CS_MODE_BIG_ENDIAN:
                 gadgets = [
@@ -122,7 +126,8 @@ class Gadgets(object):
                           ]
             arch_mode = 0
 
-        elif arch == CS_ARCH_ARM:    gadgets = []            # ARM doesn't contains RET instruction set. Only JOP gadgets
+        elif arch == CS_ARCH_ARM:
+            gadgets = []  # ARM doesn't have RET instructions. Only JOP gadgets
         elif arch == CS_ARCH_ARM64:
             if arch_endian == CS_MODE_BIG_ENDIAN:
                 gadgets = [
@@ -134,22 +139,30 @@ class Gadgets(object):
                           ]
             arch_mode = CS_MODE_ARM
 
+        elif arch == CS_ARCH_RISCV:
+            if arch_endian == CS_MODE_BIG_ENDIAN:
+                gadgets = [
+                               [b"\x80\x82", 2, 1], # c.ret
+                          ]
+            else:
+                gadgets = [
+                               [b"\x82\x80", 2, 1], # c.ret
+                          ]
+            arch_mode = CS_MODE_RISCV64 | CS_MODE_RISCVC
+
         else:
             print("Gadgets().addROPGadgets() - Architecture not supported")
             return None
 
-        if len(gadgets) > 0 :
+        if gadgets:
             return self.__gadgetsFinding(section, gadgets, arch, arch_mode + arch_endian)
         return gadgets
 
-
     def addJOPGadgets(self, section):
         arch = self.__binary.getArch()
         arch_mode = self.__binary.getArchMode()
         arch_endian = self.__binary.getEndian()
 
-
-
         if arch  == CS_ARCH_X86:
             # we start with x86 and x64 common sequences operating on registers
             gadgets = [
@@ -208,8 +221,6 @@ class Gadgets(object):
                                [b"\xf2\xff[\x10\x11\x12\x13\x16\x17]{1}", 3, 1],     # jmp  [reg]
                                [b"\xf2\xff[\xd0\xd1\xd2\xd3\xd4\xd6\xd7]{1}", 3, 1]  # call [reg]
                        ]
-
-
         elif arch == CS_ARCH_MIPS:
             if arch_endian == CS_MODE_BIG_ENDIAN:
                 gadgets = [
@@ -233,7 +244,8 @@ class Gadgets(object):
                                [b"[\x00-\xff]{3}[\x0c-\x0f][\x00-\xff]{4}", 8, 4],                            # jal addr
                                [b"[\x00-\xff]{3}[\x08-\x0b][\x00-\xff]{4}", 8, 4]                             # j addr
                           ]
-        elif arch == CS_ARCH_PPC:    gadgets = [] # PPC architecture doesn't contains reg branch instruction
+        elif arch == CS_ARCH_PPC:
+            gadgets = []  # PPC doesn't have reg branch instructions
         elif arch == CS_ARCH_SPARC:
             if arch_endian == CS_MODE_BIG_ENDIAN:
                 gadgets = [
@@ -248,12 +260,12 @@ class Gadgets(object):
             if arch_endian == CS_MODE_BIG_ENDIAN:
                 gadgets = [
                                [b"\xd6[\x1f\x5f]{1}[\x00-\x03]{1}[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}", 4, 4],  # br reg
-                               [b"\xd6\?[\x00-\x03]{1}[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}", 4, 4]  # blr reg
+                               [b"\xd6\?[\x00-\x03]{1}[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}", 4, 4]  # blr reg # noqa: W605 # FIXME: \?
                           ]
             else:
                 gadgets = [
                                [b"[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}[\x00-\x03]{1}[\x1f\x5f]{1}\xd6", 4, 4],  # br reg
-                               [b"[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}[\x00-\x03]{1}\?\xd6", 4, 4]  # blr reg
+                               [b"[\x00\x20\x40\x60\x80\xa0\xc0\xe0]{1}[\x00-\x03]{1}\?\xd6", 4, 4]  # blr reg # noqa: W605 # FIXME: \?
                           ]
             arch_mode = CS_MODE_ARM
         elif arch == CS_ARCH_ARM:
@@ -285,11 +297,41 @@ class Gadgets(object):
                                [b"[\x00-\xff][\x80-\xff][\x10-\x1e\x30-\x3e\x50-\x5e\x70-\x7e\x90-\x9e\xb0-\xbe\xd0-\xde\xf0-\xfe][\xe8\xe9]", 4, 4] # ldm {,pc}
                               ]
                 arch_mode = CS_MODE_ARM
+        elif arch == CS_ARCH_RISCV:
+            if arch_endian == CS_MODE_BIG_ENDIAN:
+                    gadgets = [
+                                #32 bits encoded register   
+                                [b"[\x00-\xff]{2}[\x00-\xff][\x67\x6f\xe7\xef]",4 , 1],
+                                [b"[\x00-\xff]{2}[\x00-\xff][\x63\xe3]", 4 , 1],
+
+                                #16bits encoded register
+                                [b"[\xa0-\xff]{1}[\xfd\xf9\xf5\xf1\xed\xe9\xe5\xe1\xdd\xd9\xd5\xd1\xcd\xc9\xc5\xc1\xbd\xb9\xb5\xb1\xad\xa9\xa5\xa1]", 2, 1], # c.j | c.beqz | c.bnez
+                                [b"[\xa0-\xff]{1}[\x0d\x09\x05\x01\x1d\x19\x15\x11\x2d\x29\x25\x21\x3d\x39\x35\x31\x4d\x49\x45\x41\x5d\x59\x55\x51]", 2, 1], # c.j | c.beqz | c.bnez
+                                [b"[\xa0-\xff]{1}[\x6d\x69\x65\x61\x7d\x79\x75\x71\x8d\x89\x85\x81\x9d\x99\x95\x91]", 2, 1], # c.j | c.beqz| c.bnez
+                                [b"[\x81-\x8f]{1}[\x02\x82]", 2, 1], #c.jr register
+                                [b"[\x91-\x9f]{1}[\x02\x82]", 2, 1], #c.jalr register
+
+                              ]
+            else:
+                    gadgets = [
+                              #32 bits encoded register
+                              [b"[\x67\x6f\xe7\xef][\x00-\xff][\x00-\xff]{2}" , 4, 1], #jalr/j/jal register, offset
+                              [b"[\x63\xe3][\x00-\xff][\x00-\xff]{2}" , 4, 1], #branch register, offset
+
+                              #16 bits encoded register
+                              [b"[\xfd\xf9\xf5\xf1\xed\xe9\xe5\xe1\xdd\xd9\xd5\xd1\xcd\xc9\xc5\xc1\xbd\xb9\xb5\xb1\xad\xa9\xa5\xa1][\xa0-\xff]{1}", 2, 1], # c.j | c.beqz | c.bnez
+                              [b"[\x0d\x09\x05\x01\x1d\x19\x15\x11\x2d\x29\x25\x21\x3d\x39\x35\x31\x4d\x49\x45\x41\x5d\x59\x55\x51][\xa0-\xff]{1}", 2, 1], # c.j | c.beqz | c.bnez
+                              [b"[\x6d\x69\x65\x61\x7d\x79\x75\x71\x8d\x89\x85\x81\x9d\x99\x95\x91][\xa0-\xff]{1}", 2, 1], # c.j | c.beqz| c.bnez
+                              [b"[\x02\x82][\x81-\x8f]{1}", 2, 1], #c.jr register
+                              [b"[\x02\x82][\x91-\x9f]{1}", 2, 1], #c.jalr register
+
+                              ]
+            arch_mode = CS_MODE_RISCV64 | CS_MODE_RISCVC
         else:
             print("Gadgets().addJOPGadgets() - Architecture not supported")
             return None
 
-        if len(gadgets) > 0 :
+        if gadgets:
             return self.__gadgetsFinding(section, gadgets, arch, arch_mode + arch_endian)
         return gadgets
 
@@ -320,9 +362,12 @@ class Gadgets(object):
                 gadgets = [
                                [b"\x0c\x00\x00\x00", 4, 4] # syscall
                           ]
-        elif arch == CS_ARCH_PPC:    gadgets = [] # TODO (sc inst)
-        elif arch == CS_ARCH_SPARC:  gadgets = [] # TODO (ta inst)
-        elif arch == CS_ARCH_ARM64:  gadgets = [] # TODO
+        elif arch == CS_ARCH_PPC:
+            gadgets = [] # TODO (sc inst)
+        elif arch == CS_ARCH_SPARC:
+            gadgets = [] # TODO (ta inst)
+        elif arch == CS_ARCH_ARM64:
+            gadgets = [] # TODO
         elif arch == CS_ARCH_ARM:
             if self.__options.thumb or self.__options.rawMode == "thumb":
                 gadgets = [
@@ -334,17 +379,27 @@ class Gadgets(object):
                                [b"\x00-\xff]{3}\xef", 4, 4] # FIXME: svc
                           ]
                 arch_mode = CS_MODE_ARM
+        elif arch == CS_ARCH_RISCV:
+
+            if arch_endian == CS_MODE_BIG_ENDIAN:
+                gadgets = [
+                               [b"\x00\x00\x00\x73", 4, 2] # syscall
+                          ]
+            else:
+                gadgets = [
+                               [b"\x73\x00\x00\x00", 4, 2] # syscall
+                          ]
+            
+            arch_mode = CS_MODE_RISCV64 | CS_MODE_RISCVC
         else:
             print("Gadgets().addSYSGadgets() - Architecture not supported")
             return None
 
-        if len(gadgets) > 0 :
+        if gadgets:
             return self.__gadgetsFinding(section, gadgets, arch, arch_mode + arch_endian)
         return []
 
-
     def passClean(self, decodes):
-
         if not decodes:
             return True
 
@@ -355,4 +410,3 @@ class Gadgets(object):
             return True
 
         return False
-
